@@ -26,4 +26,4 @@
 # activate EnPT Python environment
 source $ENPT_PYENV_ACTIVATION enpt
 
-enpt_cli.py "$@"
+enpt "$@"
