@@ -3,7 +3,7 @@
 
 # enpt_enmapboxapp, A QGIS EnMAPBox plugin providing a GUI for the EnMAP processing tools (EnPT)
 #
-# Copyright (C) 2019  Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
+# Copyright (C) 2018-2021 Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
 #
 # This software was developed within the context of the EnMAP project supported
 # by the DLR Space Administration with funds of the German Federal Ministry of
@@ -33,7 +33,9 @@ from unittest import TestCase, skipIf
 from qgis.core import QgsProcessingAlgorithm, QgsProcessingContext, QgsProcessingFeedback, QgsProcessingProvider, NULL
 
 from enmapbox.testing import initQgisApplication
-from enpt_enmapboxapp.enpt_enmapboxapp import EnPTAlgorithm, EnPTEnMAPBoxApp, EnMAPBoxApplication
+from enpt_enmapboxapp.enpt_enmapboxapp import EnPTEnMAPBoxApp, EnMAPBoxApplication
+from enpt_enmapboxapp.enpt_algorithm import EnPTAlgorithm
+from enpt_enmapboxapp.enpt_external_algorithm import ExternalEnPTAlgorithm
 # from enpt_enmapboxapp.enpt_enmapboxapp import ExampleAppGUI
 
 # initialize the QGIS API + several background states
@@ -52,12 +54,14 @@ enpt_test_parameters = dict(
     deadpix_P_algorithm='spectral',
     deadpix_P_interp_spectral='linear',
     deadpix_P_interp_spatial='linear',
+    drop_bad_bands=True,
     disable_progress_bars=False,
-    enable_cloud_screening=False,
+    output_format='GTiff',
     enable_keystone_correction=False,
     enable_vnir_swir_coreg=False,
     json_config=None,
     n_lines_to_append=NULL,
+    mode_ac='combined',
     ortho_resampAlg='bilinear',
     vswir_overlap_algorithm='vnir_only',
     output_dir='TEMPORARY_OUTPUT',
@@ -73,6 +77,7 @@ enpt_test_parameters = dict(
     scale_factor_boa_ref=10000,
     scale_factor_toa_ref=10000,
     sicor_cache_dir=None,
+    target_projection_type='UTM',
     working_dir=None)
 
 
@@ -93,38 +98,52 @@ class TestExampleEnMAPBoxApp(TestCase):
     #     self.assertEqual(kwds['foo'], 'bar')
 
     def test_processingAlgorithms(self):
-        os.environ['IS_ENPT_GUI_TEST'] = '1'
+        for algname, ALG in zip(['EnPTAlgorithm', 'ExternalEnPTAlgorith'],
+                                [EnPTAlgorithm, ExternalEnPTAlgorithm]):
 
-        alg = EnPTAlgorithm()
-        self.assertIsInstance(alg, QgsProcessingAlgorithm)
+            with self.subTest(ALG=algname):
+                try:
+                    os.environ['IS_ENPT_GUI_TEST'] = '1'
 
-        alg2 = alg.createInstance()
-        self.assertIsInstance(alg2, QgsProcessingAlgorithm)
+                    alg = ALG()
+                    self.assertIsInstance(alg, QgsProcessingAlgorithm)
 
-        with TemporaryDirectory() as td:
-            params = enpt_test_parameters.copy()
-            params['output_dir'] = td
-            params['path_l1b_enmap_image'] = os.path.join('dummy', 'path', 'to', 'EnMAP_file.zip')
-            params['path_dem'] = os.path.join('dummy', 'path', 'to', 'DEM.bsq')
+                    alg2 = alg.createInstance()
+                    self.assertIsInstance(alg2, QgsProcessingAlgorithm)
 
-            outputs = alg.processAlgorithm(params,
-                                           QgsProcessingContext(),
-                                           QgsProcessingFeedback())
+                    with TemporaryDirectory() as td:
+                        params = enpt_test_parameters.copy()
+                        params['output_dir'] = td
+                        params['path_l1b_enmap_image'] = os.path.join('dummy', 'path', 'to', 'EnMAP_file.zip')
+                        params['path_dem'] = os.path.join('dummy', 'path', 'to', 'DEM.bsq')
 
-            self.assertIsInstance(outputs, dict)
-            self.assertTrue(outputs['success'] is True,
-                            'EnPT could not be called or did not output the expected results.')
+                        # call the EnPT_Controller with the above params, it will pickle.dump them to disk
+                        outputs = alg.processAlgorithm(params,
+                                                       QgsProcessingContext(),
+                                                       QgsProcessingFeedback())
 
-            with open(os.path.join(td, 'received_args_kwargs.pkl'), 'rb') as inF:
-                content = pickle.load(inF)
+                        self.assertIsInstance(outputs, dict)
 
-            none_params = [k for k, v in params.items() if params[k] in [None, NULL]]
-            for k, v in params.items():
-                if k not in ['anaconda_root', 'json_config'] + none_params:
-                    self.assertTrue(k in content['kwargs'], "Missing key '%s' in received parameters." % k)
-                    self.assertEqual(v, content['kwargs'][k])
+                        # unpickle the dumped args/kwargs and validate that EnPT received them correctly
+                        content = None
+                        try:
+                            with open(os.path.join(td, 'received_args_kwargs.pkl'), 'rb') as inF:
+                                content = pickle.load(inF)
+                        except FileNotFoundError:
+                            pass
 
-        del os.environ['IS_ENPT_GUI_TEST']
+                        self.assertIsNotNone(content, 'The EnPT_Controller did not dump the '
+                                                      'received arguments or did not receive any.')
+
+                        none_params = [k for k, v in params.items() if params[k] in [None, NULL]]
+                        for k, v in params.items():
+                            if k not in ['anaconda_root', 'json_config'] + none_params:
+                                self.assertTrue(k in content['kwargs'],
+                                                f"Missing key '{k}' in received parameters.")
+                                self.assertEqual(v, content['kwargs'][k])
+
+                finally:
+                    del os.environ['IS_ENPT_GUI_TEST']
 
     # def test_dialog(self):
     #     """
